@@ -9,9 +9,10 @@ hierarchical graph (aka an assembly).
 All links are bidirectional.
 """
 
+from typing import Optional, Union, Any
 import os
 import collections
-import pygraphviz
+import pygraphviz  # type: ignore[import]
 from .Device import *
 
 
@@ -23,7 +24,7 @@ class DeviceGraph:
     This implements an AHP (Attributed Hierarchical Port) graph.
     """
 
-    def __init__(self, attr: dict = None) -> None:
+    def __init__(self, attr: dict[str, Any] = None) -> None:
         """
         Define an empty DeviceGraph.
 
@@ -32,16 +33,16 @@ class DeviceGraph:
         not intemediate graphs (e.g., assemblies).
         The dictionary of links uses a frozenset of DevicePorts as the key
         """
-        self.expanding = None
-        self.attr = attr if attr is not None else dict()
-        self.devices = set()
-        self.names = set()
-        self.links = dict()
-        self.ports = set()
+        self.expanding: Optional[Device] = None
+        self.attr: dict[str, Any] = attr if attr is not None else dict()
+        self.devices: set[Device] = set()
+        self.names: set[str] = set()
+        self.links: dict[frozenset[DevicePort], str] = dict()
+        self.ports: set[DevicePort] = set()
 
     def __repr__(self) -> str:
         """Pretty print a DeviceGraph with Devices followed by links."""
-        lines = list()
+        lines: list[str] = list()
         for device in self.devices:
             lines.append(str(device))
         for p0, p1 in self.links:
@@ -78,7 +79,7 @@ class DeviceGraph:
                 p1.link = p2
                 self.ports.discard(p0)
                 self.ports.add(p1)
-                latency = self.links.pop(frozenset({p0, p2}))
+                latency: str = self.links.pop(frozenset({p0, p2}))
                 # add the other device to the graph
                 self.add(p1.device)
                 self.links[frozenset({p1, p2})] = latency
@@ -145,14 +146,14 @@ class DeviceGraph:
         for (dev, _, _) in device.subs:
             self.add(dev, True)
 
-    def count_devices(self) -> dict:
+    def count_devices(self) -> dict[str, int]:
         """
         Count the Devices in a graph.
 
         Return a map of Devices to integer counts. The keys are of the
         form "CLASS_MODEL".
         """
-        counter = collections.defaultdict(int)
+        counter: dict[str, int] = collections.defaultdict(int)
         for device in self.devices:
             counter[device.get_category()] += 1
         return counter
@@ -160,14 +161,14 @@ class DeviceGraph:
     @staticmethod
     def check_port_types(p0: DevicePort, p1: DevicePort) -> bool:
         """Check that the port types for the two ports match."""
-        t0 = p0.device.portinfo[p0.name][1]
-        t1 = p1.device.portinfo[p1.name][1]
+        t0: Optional[str] = p0.device.portinfo[p0.name][1]
+        t1: Optional[str] = p1.device.portinfo[p1.name][1]
         return t0 == t1
 
     def verify_links(self) -> None:
         """Verify that all required ports are linked up."""
         # Create a map of Devices to all ports linked on those Devices.
-        d2ports = collections.defaultdict(set)
+        d2ports: dict[Device, set[str]] = collections.defaultdict(set)
         for p0, p1 in self.links:
             d2ports[p0.device].add(p0.name)
             d2ports[p1.device].add(p1.name)
@@ -196,13 +197,13 @@ class DeviceGraph:
         """
         self.check_partition()
         while True:
-            devices = set()
-            linksToRemove = list()
+            devices: set[Device] = set()
+            linksToRemove: list[frozenset[DevicePort]] = list()
             for p0, p1 in self.links:
                 # One of the Devices is on the rank that we are
                 # following links on
-                if (p0.device.partition[0] == rank
-                        or p1.device.partition[0] == rank):
+                if (p0.device.partition[0] == rank  # type: ignore[index]
+                        or p1.device.partition[0] == rank):  # type: ignore[index]
                     for p in [p0, p1]:
                         if p.device.library is None:
                             devices.add(p.device)
@@ -227,7 +228,7 @@ class DeviceGraph:
                 break
 
     def flatten(self, levels: int = None, name: str = None,
-                rank: int = None, expand: set = None) -> None:
+                rank: int = None, expand: set[Device] = None) -> None:
         """
         Recursively flatten the graph by the specified number of levels.
 
@@ -251,7 +252,7 @@ class DeviceGraph:
         if levels == 0:
             return
 
-        assemblies = set()
+        assemblies: set[Device] = set()
         if name is not None:
             splitName = name.split(".")
 
@@ -262,7 +263,7 @@ class DeviceGraph:
             devs = self.devices
 
         for dev in devs:
-            assembly = dev.library is None
+            assembly: bool = dev.library is None
             if not assembly:
                 continue
 
@@ -271,7 +272,7 @@ class DeviceGraph:
                 assembly &= splitName == dev.name.split(".")[0: len(splitName)]
             # rank to check
             if rank is not None:
-                assembly &= rank == dev.partition[0]
+                assembly &= rank == dev.partition[0]  # type: ignore[index]
 
             if assembly:
                 assemblies.add(dev)
@@ -282,7 +283,7 @@ class DeviceGraph:
         # Expand the required Devices
         for device in assemblies:
             self.expanding = device
-            device.expand(self)
+            device.expand(self)  # type: ignore[operator, arg-type]
             self.names.discard(device.name)
             self.devices.discard(device)
         self.expanding = None
@@ -292,11 +293,11 @@ class DeviceGraph:
             self.flatten(None if levels is None else levels-1, name, rank)
 
     def write_dot(self, name: str, draw: bool = False, ports: bool = False,
-                  hierarchy: bool = True, directory: str = 'output') -> None:
+                  hierarchy: bool = True) -> None:
         """
         Take a DeviceGraph and write it as a graphviz dot graph.
 
-        All output will be stored in a folder specified by the directory parameter
+        All output will be stored in a folder called output
         The draw parameter will automatically generate SVGs if set to True
         The ports parameter will display ports on the graph if set to True
 
@@ -305,18 +306,17 @@ class DeviceGraph:
         view of the graph as it is.
         hierarchy is True by default, and highly recommended for large graphs
         """
-        if not os.path.exists(directory):
-            os.makedirs(directory)
+        if not os.path.exists('output'):
+            os.makedirs('output')
 
         if hierarchy:
-            self.__write_dot_hierarchy(name, draw, ports, directory)
+            self.__write_dot_hierarchy(name, draw, ports)
         else:
-            self.__write_dot_flat(name, draw, ports, directory)
+            self.__write_dot_flat(name, draw, ports)
 
     def __write_dot_hierarchy(self, name: str, draw: bool = False,
-                              ports: bool = False, directory: str = 'output',
-                              assembly: str = None,
-                              types: set = None) -> set:
+                              ports: bool = False, assembly: str = None,
+                              types: set[str] = None) -> set[str]:
         """
         Take a DeviceGraph and write dot files for each assembly.
 
@@ -325,7 +325,7 @@ class DeviceGraph:
         assembly and types should NOT be specified by the user, they are
         soley used for recursion of this function
         """
-        graph = self.__format_graph(name, ports, directory)
+        graph: pygraphviz.AGraph = self.__format_graph(name, ports)
         if types is None:
             types = set()
 
@@ -338,14 +338,14 @@ class DeviceGraph:
         # Expand all unique assembly types and write separate graphviz files
         for dev in self.devices:
             if dev.library is None:
-                category = dev.get_category()
+                category: str = dev.get_category()
                 if category not in types:
                     types.add(category)
                     expanded = DeviceGraph()
-                    dev.expand(expanded)
+                    dev.expand(expanded)  # type: ignore[operator, arg-type]
                     types = expanded.__write_dot_hierarchy(category, draw,
-                                                           ports, directory,
-                                                           dev.name, types)
+                                                           ports, dev.name,
+                                                           types)
 
         # Need to check if the provided assembly name is in the graph
         # and if so make that our cluster
@@ -394,20 +394,21 @@ class DeviceGraph:
 
         self.__dot_add_links(graph, ports, assembly, splitName, splitNameLen)
 
-        graph.write(f"{directory}/{name}.dot")
+        graph.write(f"output/{name}.dot")
         if draw:
-            graph.draw(f"{directory}/{name}.svg", format='svg', prog='dot')
+            graph.draw(f"output/{name}.svg", format='svg', prog='dot')
 
         return types
 
     def __write_dot_flat(self, name: str, draw: bool = False,
-                         ports: bool = False, directory: str = 'output') -> None:
+                         ports: bool = False) -> None:
         """
         Write the DeviceGraph as a DOT file.
 
         It is suggested that you use write_dot_hierarchy for large graphs
         """
-        graph = self.__format_graph(name, ports, directory)
+        graph: pygraphviz.AGraph = self.__format_graph(name, ports)
+
         for dev in self.devices:
             label = dev.name
             if dev.model is not None:
@@ -424,13 +425,12 @@ class DeviceGraph:
 
         self.__dot_add_links(graph, ports)
 
-        graph.write(f"{directory}/{name}.dot")
+        graph.write(f"output/{name}.dot")
         if draw:
-            graph.draw(f"{directory}/{name}.svg", format='svg', prog='dot')
+            graph.draw(f"output/{name}.svg", format='svg', prog='dot')
 
     @staticmethod
-    def __format_graph(name: str, record: bool = False,
-                       directory: str = 'output') -> pygraphviz.AGraph:
+    def __format_graph(name: str, record: bool = False) -> pygraphviz.AGraph:
         """Format a new graph."""
         h = ('.edge:hover text {\n'
              '\tfill: red;\n'
@@ -439,11 +439,11 @@ class DeviceGraph:
              '\tstroke: red;\n'
              '\tstroke-width: 10;\n'
              '}')
-        if not os.path.exists(f"{directory}/highlightStyle.css"):
-            with open(f"{directory}/highlightStyle.css", 'w') as f:
+        if not os.path.exists('output/highlightStyle.css'):
+            with open('output/highlightStyle.css', 'w') as f:
                 f.write(h)
 
-        graph = pygraphviz.AGraph(strict=False, name=name)
+        graph: pygraphviz.AGraph = pygraphviz.AGraph(strict=False, name=name)
         graph.graph_attr['stylesheet'] = 'highlightStyle.css'
         graph.node_attr['style'] = 'filled'
         graph.node_attr['fillcolor'] = '#EEEEEE'  # light gray fill
@@ -454,11 +454,11 @@ class DeviceGraph:
 
         return graph
 
-    def __dot_add_links(self, graph, ports: bool = False,
-                        assembly: str = None, splitName: list = None,
+    def __dot_add_links(self, graph: pygraphviz.AGraph, ports: bool = False,
+                        assembly: str = None, splitName: list[str] = None,
                         splitNameLen: int = None) -> None:
         """Add edges to the graph with a label for the number of edges."""
-        def port2Node(port: DevicePort) -> str:
+        def port2Node(port: DevicePort) -> Union[str, tuple[str, str]]:
             """Return a node name given a DevicePort."""
             node = port.device.name
             if node == assembly:
@@ -472,7 +472,8 @@ class DeviceGraph:
                 return node
 
         # Create a list of all of the links
-        links = list()
+        nodeType = Union[str, tuple[str, str]]
+        links: list[tuple[nodeType, nodeType]] = list()
         for p0, p1 in self.links:
             links.append(frozenset({port2Node(p0), port2Node(p1)}))
 
