@@ -2,6 +2,7 @@
 
 """This module extends a DeviceGraph to enable SST Simulation output."""
 
+from typing import Optional, Any
 import os
 import orjson
 from .Device import *
@@ -18,12 +19,12 @@ class SSTGraph(DeviceGraph):
 
     def __init__(self, graph: DeviceGraph) -> None:
         """Point at the DeviceGraph variables."""
-        self.expanding = None
-        self.attr = graph.attr
-        self.devices = graph.devices
-        self.names = graph.names
-        self.links = graph.links
-        self.ports = graph.ports
+        self.expanding: Optional[Device] = None
+        self.attr: dict[str, Any] = graph.attr
+        self.devices: set[Device] = graph.devices
+        self.names: set[str] = graph.names
+        self.links: dict[frozenset[DevicePort], str] = graph.links
+        self.ports: set[DevicePort] = graph.ports
 
     def build(self, nranks: int = 1) -> dict:
         """
@@ -36,7 +37,7 @@ class SSTGraph(DeviceGraph):
         setting nranks in this function to the total number of ranks used
         """
         # only import sst when we are going to build the graph
-        import sst
+        import sst  # type: ignore[import]
 
         # If this is serial or sst is doing the partitioning,
         # generate the entire graph
@@ -62,13 +63,12 @@ class SSTGraph(DeviceGraph):
             self.follow_links(rank, True)
             return self.__build_model(True)
 
-    def write_json(self, name: str, nranks: int = 1,
-                   directory: str = 'output',
-                   program_options: dict = None) -> None:
+    def write_json(self, filename: str, nranks: int = 1,
+                   program_options: dict[str, Any] = None) -> None:
         """
         Generate the JSON and write it to the specified filename.
 
-        All output will be stored in a folder specified by the directory parameter
+        All output will be stored in a folder called output
         The program_options dictionary provides a way to pass SST
         program options, such as timebase and stopAtCycle.
 
@@ -77,8 +77,8 @@ class SSTGraph(DeviceGraph):
         can do this by using the Device.set_partition() function and then
         setting nranks in this function to the total number of ranks used
         """
-        if not os.path.exists(directory):
-            os.makedirs(directory)
+        if not os.path.exists('output'):
+            os.makedirs('output')
 
         self.flatten()
         self.verify_links()
@@ -86,19 +86,22 @@ class SSTGraph(DeviceGraph):
         # If this is serial or sst is doing the partitioning,
         # just write the whole thing
         if nranks == 1:
-            self.__write_model(f"{directory}/{name}.json", nranks,
+            self.__write_model(f"output/{filename}", nranks,
                                self.devices, self.links, program_options)
 
         # Write a JSON file for each rank
         else:
+            (base, ext) = os.path.splitext(f"output/{filename}")
             self.check_partition()
             partition = self.__partition_graph(nranks)
             for rank in range(nranks):
-                self.__write_model(f"{directory}/{name}{rank}.json", nranks,
+                self.__write_model(base + str(rank) + ext, nranks,
                                    partition[rank][0], partition[rank][1],
                                    program_options)
 
-    def __partition_graph(self, nranks: int) -> list:
+    pType = list[tuple[set[Device], dict[frozenset, str]]]
+
+    def __partition_graph(self, nranks: int) -> pType:
         """
         Store all the devices and links for a given rank.
 
@@ -107,7 +110,7 @@ class SSTGraph(DeviceGraph):
         It is faster to partition the graph once rather than search it
         O(p) times.
         """
-        partition = [(set(), dict()) for p in range(nranks)]
+        partition: SSTGraph.pType = [(set(), dict()) for p in range(nranks)]
 
         for p0, p1 in self.links:
             link = frozenset({p0, p1})
@@ -118,8 +121,8 @@ class SSTGraph(DeviceGraph):
                 d0 = d0.subOwner
             while d1.subOwner is not None:
                 d1 = d1.subOwner
-            r0 = d0.partition[0]
-            r1 = d1.partition[0]
+            r0 = d0.partition[0]  # type: ignore[index]
+            r1 = d1.partition[0]  # type: ignore[index]
 
             partition[r0][0].add(d0)
             partition[r0][0].add(d1)
@@ -131,8 +134,8 @@ class SSTGraph(DeviceGraph):
         return partition
 
     @staticmethod
-    def __encode(attr: dict,
-                 stringify: bool = False) -> dict:
+    def __encode(attr: dict[str, Any],
+                 stringify: bool = False) -> dict[str, Any]:
         """
         Convert attributes into SST Params.
 
@@ -142,11 +145,11 @@ class SSTGraph(DeviceGraph):
         Ignore bad conversions.
         """
 
-        def supported_f(x) -> bool:
+        def supported_f(x: Any) -> bool:
             """Return whether the type is supported by SST."""
             return isinstance(x, (bool, float, int, str))
 
-        params = dict()
+        params: dict[str, Any] = dict()
         for (key, val) in attr.items():
             native = supported_f(val)
             if not native and isinstance(val, list):
@@ -159,19 +162,21 @@ class SSTGraph(DeviceGraph):
             else:
                 try:
                     # serialize the value to json bytes,
-                    # then decode to a string
-                    b = orjson.dumps(val, option=orjson.OPT_INDENT_2)
-                    params[key] = b.decode('utf-8')
+                    # then deserialize into a python dict
+                    params[key] = orjson.loads(
+                        orjson.dumps(val, option=orjson.OPT_INDENT_2)
+                    )
                 except Exception:
                     pass
 
         return params
 
-    def __build_model(self, self_partition: bool) -> dict:
+    def __build_model(self, self_partition: bool
+                      ) -> dict[str, 'sst.Component']:  # type: ignore[name-defined]
         """Generate the model for the SST program."""
         # only import sst when we are going to build the graph inside of sst
         import sst
-        n2c = dict()
+        n2c: dict[str, 'sst.Component'] = dict()
 
         # Set up global parameters.
         global_params = self.__encode(self.attr)
@@ -187,8 +192,8 @@ class SSTGraph(DeviceGraph):
                     c1 = comp.setSubComponent(n1, d1.library)
                 else:
                     c1 = comp.setSubComponent(n1, d1.library, s1)
-                d1.attr.update({'type': d1.type, 'model': d1.model})
-                c1.addParams(self.__encode(d1.attr))
+                c1.addParams(self.__encode(
+                    {'type': d1.type, 'model': d1.model} | d1.attr))
                 n2c[d1.name] = c1
                 for key in global_params:
                     c1.addGlobalParamSet(key)
@@ -200,13 +205,13 @@ class SSTGraph(DeviceGraph):
         for d0 in self.devices:
             if d0.subOwner is None and d0.library is not None:
                 c0 = sst.Component(d0.name, d0.library)
-                d0.attr.update({'type': d0.type, 'model': d0.model})
-                c0.addParams(self.__encode(d0.attr))
+                c0.addParams(self.__encode(
+                    {'type': d0.type, 'model': d0.model} | d0.attr))
                 # Set the component partition if we are self-partitioning
                 if self_partition:
-                    thread = (0 if d0.partition[1] is None
-                              else d0.partition[1])
-                    c0.setRank(d0.partition[0], thread)
+                    thread = (0 if d0.partition[1] is None  # type: ignore[index]
+                              else d0.partition[1])  # type: ignore[index]
+                    c0.setRank(d0.partition[0], thread)  # type: ignore[index]
                 n2c[d0.name] = c0
                 for key in global_params:
                     c0.addGlobalParamSet(key)
@@ -230,11 +235,11 @@ class SSTGraph(DeviceGraph):
         # Return a map of component names to components.
         return n2c
 
-    def __write_model(self, filename: str, nranks: int, devices: set,
-                      links: dict,
-                      program_options: dict = None) -> None:
+    def __write_model(self, filename: str, nranks: int, devices: set[Device],
+                      links: dict[frozenset[DevicePort], str],
+                      program_options: dict[str, Any] = None) -> None:
         """Write this DeviceGraph out as JSON."""
-        model = dict()
+        model: dict[str, Any] = dict()
 
         # Write the program options to the model.
         if program_options is None:
@@ -253,19 +258,20 @@ class SSTGraph(DeviceGraph):
             model["global_params"][key] = dict({key: val})
         global_set = list(global_params.keys())
 
-        def recurseSubcomponents(dev: Device) -> list:
+        def recurseSubcomponents(dev: Device) -> list[Any]:
             """Add subcomponents to the Device."""
             subcomponents = list()
             for (d1, n1, s1) in dev.subs:
                 if d1.library is None:
                     raise RuntimeError(f"No library: {d1.name}")
-                
-                d1.attr.update({'type': d1.type, 'model': d1.model})
+
                 item = {
                         "slot_name": n1,
                         "type": d1.library,
                         "slot_number": s1,
-                        "params": self.__encode(d1.attr, True),
+                        "params": self.__encode(
+                            {'type': d1.type, 'model': d1.model} | d1.attr,
+                            True),
                         "params_global_sets": global_set,
                     }
                 if len(d1.subs) > 0:
@@ -278,18 +284,19 @@ class SSTGraph(DeviceGraph):
         components = list()
         for d0 in devices:
             if d0.subOwner is None and d0.library is not None:
-                d0.attr.update({'type': d0.type, 'model': d0.model})
                 component = {
                     "name": d0.name,
                     "type": d0.library,
-                    "params": self.__encode(d0.attr, True),
+                    "params": self.__encode(
+                        {'type': d0.type, 'model': d0.model} | d0.attr,
+                        True),
                     "params_global_sets": global_set,
                 }
                 if nranks > 1:
                     component["partition"] = {
-                        "rank": d0.partition[0],
-                        "thread": (0 if d0.partition[1] is None
-                                   else d0.partition[1]),
+                        "rank": d0.partition[0],  # type: ignore[index]
+                        "thread": (0 if d0.partition[1] is None  # type: ignore[index]
+                                   else d0.partition[1]),  # type: ignore[index]
                     }
 
                 subcomponents = recurseSubcomponents(d0)
